﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
namespace TechAmit.SharePoint.LogicalFieldUpdate
{
    public class ConditionalTest
    {
        public enum TestCondition
        {
            None,
            Equal,
            NotEqual,
            Greater,
            GreaterOrEqual,
            Lesser,
            LesserOrEqual,
            StartsWith,
            NotStartsWith,
            EndsWith,
            NotEndsWith,
            Contains,
            NotContains,
            // Between,
            RegexMatches,
            RegexNotMatches,
            Null,
            NotNull,
            NullOrEmpty,
            AnythingOrNull,
            AnythingNotNull
        }

        public enum TestDataType
        {
            Numeric,
            String,
            DateTime,
            None
        }

        public class TestOptions
        {
            public StringComparison StringComparison;
            public RegexOptions RegexOptions;
        }

        public static ConditionalTest.TestCondition ParseCondition(string value)
        {
            switch (value)
            {
                case Constants.ConditionTitle.Equal:
                    return ConditionalTest.TestCondition.Equal;

                case Constants.ConditionTitle.NotEqual:
                    return ConditionalTest.TestCondition.NotEqual;

                case Constants.ConditionTitle.Greater:
                    return ConditionalTest.TestCondition.Greater;

                case Constants.ConditionTitle.GreaterOrEqual:
                    return ConditionalTest.TestCondition.GreaterOrEqual;

                case Constants.ConditionTitle.Lesser:
                    return ConditionalTest.TestCondition.Lesser;

                case Constants.ConditionTitle.LesserOrEqual:
                    return ConditionalTest.TestCondition.LesserOrEqual;

                case Constants.ConditionTitle.StartsWith:
                    return ConditionalTest.TestCondition.StartsWith;

                case Constants.ConditionTitle.NotStartsWith:
                    return ConditionalTest.TestCondition.NotStartsWith;

                case Constants.ConditionTitle.EndsWith:
                    return ConditionalTest.TestCondition.EndsWith;

                case Constants.ConditionTitle.NotEndsWith:
                    return ConditionalTest.TestCondition.NotEndsWith;

                case Constants.ConditionTitle.Contains:
                    return ConditionalTest.TestCondition.Contains;

                case Constants.ConditionTitle.NotContains:
                    return ConditionalTest.TestCondition.NotContains;

                case Constants.ConditionTitle.RegexMatches:
                    return ConditionalTest.TestCondition.RegexMatches;
                
                case Constants.ConditionTitle.RegexNotMatches:
                    return ConditionalTest.TestCondition.RegexNotMatches;

                case Constants.ConditionTitle.Null:
                    return ConditionalTest.TestCondition.Null;

                case Constants.ConditionTitle.NotNull:
                    return ConditionalTest.TestCondition.NotNull;

                case Constants.ConditionTitle.NullOrEmpty:
                    return ConditionalTest.TestCondition.NullOrEmpty;

                case Constants.ConditionTitle.AnythingButNotNull:
                    return ConditionalTest.TestCondition.AnythingNotNull;

                case Constants.ConditionTitle.AnythingIncludingNull:
                    return ConditionalTest.TestCondition.AnythingOrNull;

                default:
                    return ConditionalTest.TestCondition.None;

            }
        }

        private class TestValues
        {
            public TestValues(object value1, object value2)
            {
                // Set values
                TestValue1 = value1;
                TestValue2 = value2;

                // Check if both values are null
                BothValuesAreNull = (TestValue1 == null && TestValue2 == null);

                // Check if either value is null
                EitherValueIsNull = (TestValue1 == null || TestValue2 == null);

                // Check if none value is null ( each value is not null)
                NoneValueIsNull = (TestValue1 != null && TestValue2 != null);
            }
            public object TestValue1 { get; private set; }
            public object TestValue2 { get; private set; }
            public bool BothValuesAreNull { get; private set; }
            public bool EitherValueIsNull { get; private set; }
            public bool NoneValueIsNull { get; private set; }
        }

        public bool DoTest(TestCondition testType, object value1, object value2,TestOptions testOptions)
        {
            var testValues = new TestValues(value1, value2);

            // Figure out data type of values
            TestDataType testDataType;
            if (testType == TestCondition.RegexMatches)
            {
                // Regex only supports string data type
                testDataType = TestDataType.String;
            }
            else
            {
                testDataType = DetermineValuesDataType(testValues);
            }


            return DoTest(testType, testDataType, value1, value2,testOptions);
        }


        public bool DoTest(TestCondition testType,TestDataType valueDataType, object value1, object value2,TestOptions testOptions)
        {

            // If no test type is defined, just return false
            if (testType == TestCondition.None) return false;

            #region Null, Not Null and NullOrEmpty check

            // This tries to match anything.
            // It always return true
            if (testType == TestCondition.AnythingOrNull)
            {
                return true;
            }

            // This matches anything but null
            if (testType == TestCondition.AnythingNotNull)
            {
                return (value1 != null);
            }

            // This matches null
            if (testType == TestCondition.Null)
            {
                return (value1 == null);
            }

            // This matches not null
            if (testType == TestCondition.NotNull)
            {
                return (value1 != null);
            }

            //This matches null or empty string value
            if (testType == TestCondition.NullOrEmpty)
            {
                return (value1 == null || value1.ToString() == string.Empty);
            }
            #endregion

            // Create values object
            TestValues testValues = new TestValues(value1, value2);

            #region Rules
            // Rule: The data type should not None
            if (valueDataType == TestDataType.None)
            {
                return false;
            }

            // Rule: Each values should be not Null
            if (testValues.NoneValueIsNull == false)
            {
                return false;
            }
            #endregion

            #region Check condition according to data type
            switch (valueDataType)
            {
                case TestDataType.Numeric:
                    return NumericTest(testValues, testType);

                case TestDataType.String:
                    return StringTest(testValues, testType,testOptions);

                case TestDataType.DateTime:
                    return DateTimeTest(testValues, testType);

            }
            #endregion

            return false;
        }


        // Determine the test value data type
        private TestDataType DetermineValuesDataType(TestValues testValues)
        {
            // If both values are null
            if (testValues.BothValuesAreNull || testValues.EitherValueIsNull)
            {
                return TestDataType.None;
            }


            // Check if values are of DateTime data type
            try
            {
                DateTime v1 = Convert.ToDateTime(testValues.TestValue1);
                DateTime v2 = Convert.ToDateTime(testValues.TestValue2);

                // conversion success
                return TestDataType.DateTime;
            }
            catch { }

            // Check if values are of Numeric data type
            try
            {
                Decimal v1 = Convert.ToDecimal(testValues.TestValue1);
                Decimal v2 = Convert.ToDecimal(testValues.TestValue2);

                // conversion success
                return TestDataType.Numeric;
            }
            catch { }

            // return string data type
            return TestDataType.String;
        }

        #region Conditional test for each data type
        private bool DateTimeTest(TestValues testValues, TestCondition testCondition)
        {

            DateTime value1;
            DateTime value2;

            try
            {
                value1 = Convert.ToDateTime(testValues.TestValue1);
                value2 = Convert.ToDateTime(testValues.TestValue2);
            }
            catch
            {
                // An error occured while converting the values.
                // Logical test failed, so return false
                return false;
            }

            switch (testCondition)
            {
                case TestCondition.Equal:
                    return (value1 == value2);

                case TestCondition.Greater:
                    return (value1 > value2);

                case TestCondition.GreaterOrEqual:
                    return (value1 >= value2);

                case TestCondition.Lesser:
                    return (value1 < value2);

                case TestCondition.LesserOrEqual:
                    return (value1 <= value2);

                case TestCondition.NotEqual:
                    return (value1 != value2);

            }

            // No supported logic test found
            return false;
        }
        private bool NumericTest(TestValues testValues, TestCondition testCondition)
        {

            decimal value1;
            decimal value2;

            try
            {
                value1 = Convert.ToDecimal(testValues.TestValue1);
                value2 = Convert.ToDecimal(testValues.TestValue2);
            }
            catch
            {
                // An error occured while converting the values.
                // Logical test failed, so return false
                return false;
            }

            switch (testCondition)
            {
                case TestCondition.Equal:
                    return (value1 == value2);
                
                case TestCondition.Greater:
                    return (value1 > value2);

                case TestCondition.GreaterOrEqual:
                    return (value1 >= value2);

                case TestCondition.Lesser:
                    return (value1 < value2);

                case TestCondition.LesserOrEqual:
                    return (value1 <= value2);

                case TestCondition.NotEqual:
                    return (value1 != value2);

            }

            // No supported logic test found
            return false;
        }
        private bool StringTest(TestValues testValues, TestCondition testCondition,TestOptions testOptions)
        {

            string value1 = testValues.TestValue1.ToString();
            string value2 = testValues.TestValue2.ToString();

            switch (testCondition)
            {
                case TestCondition.Equal:
                    return (value1.Equals(value2, testOptions.StringComparison));

                case TestCondition.NotEqual:
                    return (!value1.Equals(value2, testOptions.StringComparison));

                case TestCondition.StartsWith:
                    return (value1.StartsWith(value2, testOptions.StringComparison));

                case TestCondition.NotStartsWith:
                    return (!value1.StartsWith(value2, testOptions.StringComparison));

                case TestCondition.EndsWith:
                    return (value1.EndsWith(value2, testOptions.StringComparison));

                case TestCondition.NotEndsWith:
                    return (!value1.EndsWith(value2, testOptions.StringComparison));

                case TestCondition.Contains:
                    return (value1.IndexOf(value2, testOptions.StringComparison)>=0);

                case TestCondition.NotContains:
                    return (value1.IndexOf(value2, testOptions.StringComparison)<0);

                case TestCondition.RegexMatches:
                    var regex1 = new Regex(value2, testOptions.RegexOptions);
                    return regex1.IsMatch(value1);

                case TestCondition.RegexNotMatches:
                    var regex2 = new Regex(value2, testOptions.RegexOptions);
                    return !regex2.IsMatch(value1);


            }

            // No supported logic test found
            return false;
        }

        #endregion
    }
}
