﻿using System;
using System.Text.RegularExpressions;
using System.Text;
using System.Linq;
using System.Collections.Generic;
using System.Security.Permissions;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Security;
using Microsoft.SharePoint.Utilities;
using Microsoft.SharePoint.Workflow;
namespace TechAmit.SharePoint.LogicalFieldUpdate.ListenList
{
    /// <summary>
    /// List Item Events
    /// </summary>
    public class ListenList : SPItemEventReceiver
    {



       /// <summary>
       /// An item is being added.
       /// </summary>
       public override void ItemAdding(SPItemEventProperties properties)
       {
           base.ItemAdding(properties);

           // Check if this is our logic list
           if (properties.ListTitle == Constants.LogicListName)
           {
               LogicItemValidatorHandler(properties);
           }
           else
           {
               RunLogicUpdate(properties, Constants.LogicalCheckEventsTitle.ItemAdding);
           }
       }

       /// <summary>
       /// An item is being updated.
       /// </summary>
       public override void ItemUpdating(SPItemEventProperties properties)
       {
           base.ItemUpdating(properties);

           // Ignore if source list is logical list
           if (properties.ListTitle == Constants.LogicListName)
           {
               LogicItemValidatorHandler(properties);
           }
           else
           {
               RunLogicUpdate(properties, Constants.LogicalCheckEventsTitle.ItemUpdating);
           }

       }

       /// <summary>
       /// An item was updated
       /// </summary>
       public override void ItemUpdated(SPItemEventProperties properties)
       {
           base.ItemUpdated(properties);

           // Ignore if source list is logical list
           if (properties.ListTitle == Constants.LogicListName) { return; }

           RunLogicUpdate(properties.ListItem, Constants.LogicalCheckEventsTitle.ItemUpdated);
       }

       public override void ItemAdded(SPItemEventProperties properties)
       {
           base.ItemAdded(properties);

           // Ignore if source list is logical list
           if (properties.ListTitle == Constants.LogicListName) { return; }

           RunLogicUpdate(properties.ListItem, Constants.LogicalCheckEventsTitle.ItemAdded);
       }

       private void RunLogicUpdate(SPListItem item, string eventTitle)
       {
           using (var disableScopt = new DisabledEventScope())
           {
               var logicalList = item.Web.Lists.TryGetList(Constants.LogicListName);

               // First make sure at least one logic item has event selected
               if (IsEventExist(eventTitle, item.ParentList.Title, logicalList))
               {
                   var lr = new LogicFieldUpdater(logicalList);
                   if (lr.RunLogicalUpdate(item))
                   {
                       item.SystemUpdate(false);
                   }
               }
           }

       }

       private void RunLogicUpdate(SPItemEventProperties prop, string eventTitle)
       {
           var logicalList = prop.Web.Lists.TryGetList(Constants.LogicListName);

           // First make sure at least one logic item has event selected
           if (IsEventExist(eventTitle,prop.ListTitle, logicalList))
           {
               var lr = new LogicFieldUpdater(logicalList);
               lr.RunLogicalUpdate(prop);
           }
       }

        // Validate logic list item
       private void LogicItemValidatorHandler(SPItemEventProperties prop)
       {
           // Read the parameter if we need to validate this logic item
           string validateFieldInternalName = prop.List.Fields[Constants.LogicListFieldName.Validate].InternalName;
           bool NeedToValidate = Convert.ToBoolean(prop.AfterProperties[validateFieldInternalName].ToString());

           if (NeedToValidate)
           {
               var logicValidator = new LogicValidator(prop);
               if (logicValidator.Validate() == false)
               {
                   // Validation failed for logic item

                   // Add error messages and cancel
                   prop.ErrorMessage = string.Join(Environment.NewLine, logicValidator.ErrorMessages.ToArray());
                   prop.Cancel = true;
               }
           }
       }
        // Check in logical list if the raised event source list 
        // has at least one logic item with event defined
       private bool IsEventExist(string eventTitle, string listTitle,  SPList logicalList)
       {
           string caml =    "<Where>" + 
                            "<And>" +
                                "<Eq>" + 
                                "<FieldRef Name='{0}'/>" + 
                                "<Value Type='Text'>{1}</Value>" + 
                                "</Eq>" + 
                                "<And>" + 
                                    "<Eq><FieldRef Name='{2}'/>" +
                                    "<Value Type='MultiChoice'>{3}</Value>" +
                                    "</Eq>" +
                                    "<Eq><FieldRef Name='{4}'/>" +
                                    "<Value Type='Boolean'>{5}</Value>" + // Active
                                    "</Eq>" +
                                "</And>" +
                            "</And>" +
                            "</Where>";

           object[] camlParams = {
                                     logicalList.Fields[Constants.LogicListFieldName.ListName].InternalName,
                                     SPEncode.HtmlEncode(listTitle),
                                     logicalList.Fields[Constants.LogicListFieldName.CheckEvent].InternalName,
                                     SPEncode.HtmlEncode(eventTitle),
                                     logicalList.Fields[Constants.LogicListFieldName.Active].InternalName,
                                     1,
                                 };

           var query = new SPQuery();
           query.Query = string.Format(caml,camlParams);
           query.ViewFields = "<FieldRef Name='ID' />";
           query.ViewFieldsOnly = true;

           var queryResult = logicalList.GetItems(query);

           return (queryResult != null && queryResult.Count > 0) ? true : false;
       }


    }
}
