﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Utilities;
namespace TechAmit.SharePoint.LogicalFieldUpdate
{
    public class LogicFieldUpdater
    {

        StringComparison IgnoreCase = StringComparison.CurrentCultureIgnoreCase;

        public class ListItemReader
        {
            enum ReadType
            {
                EventProp,
                ListItem
            }
            public ListItemReader(SPItemEventProperties prop)
            {
                ReadingType = ReadType.EventProp;
                EventProp = prop;
                ItemList = prop.List;
            }
            public ListItemReader(SPListItem item)
            {
                ReadingType = ReadType.ListItem;
                ListItem = item;
                ItemList = item.ParentList;
            }

            private SPItemEventProperties EventProp { get; set; }
            private SPListItem ListItem { get; set; }
            private ReadType ReadingType { get; set; }
            public SPList ItemList { get; private set; }

            private string GetFieldValue(string fldDisplayName, object value)
            {
                if (value == null) return null;
                return ItemList.Fields[fldDisplayName].GetFieldValueAsText(value);
            }
            public object GetFieldValue(string fieldName)
            {
                

                if (ReadingType == ReadType.EventProp)
                {
                    string fieldInternalName = EventProp.List.Fields[fieldName].InternalName;
                    object value = EventProp.AfterProperties[fieldInternalName];
                    return GetFieldValue(fieldName, value);
                }

                if (ReadingType == ReadType.ListItem)
                {
                    string fldInternalName = ListItem.ParentList.Fields[fieldName].InternalName;
                    return GetFieldValue(fieldName, ListItem[fieldName]);
                }

                return null;
            }
            public void FieldUpdate(string fieldName, object value)
            {
                if (string.IsNullOrEmpty(fieldName)) return;

                

                if (ReadingType == ReadType.EventProp)
                {
                    string fieldInternalName = EventProp.List.Fields[fieldName].InternalName;

                    // EventProp.AfterProperties[fieldName] = value;
                    EventProp.AfterProperties[fieldInternalName] = value;
                    return;
                }

                if (ReadingType == ReadType.ListItem)
                {
                    ListItem[fieldName] = value;
                    return;
                }
            }
        }


        public LogicFieldUpdater(SPList logicList)
        {
            LogicList = logicList;
        }

        private SPList LogicList { get; set; }


        public bool RunLogicalUpdate(SPItemEventProperties prop)
        {
            var reader = new ListItemReader(prop);
            return DoUpdate(reader);
        }

        public bool RunLogicalUpdate(SPListItem item)
        {
            var reader = new ListItemReader(item);
            return DoUpdate(reader);
        }

        private bool DoUpdate(ListItemReader reader)
        {

            // Ignore if it is running our setting list
            if (string.Equals(reader.ItemList.Title, Constants.LogicListName, IgnoreCase)) return false;


            // Get user defined logics
            var logics = ParseToLogicInfoCollection(reader);

            // Check if any logic exist
            if (logics == null || logics.Count == 0) return false;

            // Final updatable value
            bool IsUpdated = false;

            var test = new ConditionalTest();

            // Loop through each logic
            foreach (LogicInfo logic in logics)
            {
                // Check if condition matches / logic is true
                bool checkResult=false;
                if (logic.ValueType == ConditionalTest.TestDataType.None)
                {
                    // Either "Auto Detect" is defined or not a valid selection is found
                    checkResult = test.DoTest(logic.Condition, logic.ValueToMatch, logic.ValueToMatchWith, logic.TestOptions);
                }
                else
                {
                    // User defined value type is found and do the test accordingly
                    checkResult = test.DoTest(logic.Condition, logic.ValueType, logic.ValueToMatch, logic.ValueToMatchWith, logic.TestOptions);
                }

                

                if (checkResult == true)
                {
                    reader.FieldUpdate(logic.FieldNameToUpdate, logic.ValueToUpdateWith);
                    IsUpdated = true;
                }

                #region Decide if need to check next condition
                // Check if we need to stop here
                if (logic.CheckNextCondition == Constants.CheckNext.Always)
                {
                    // User has defined to check next condition always
                    // no matter if current condition is true or false
                }
                else if (logic.CheckNextCondition == Constants.CheckNext.IfTrue)
                {
                    // User has defined that proceed to next 
                    // condition check only if the current
                    // condition is turned to True.

                    // So if not matched with what the user
                    // defined just don't proceed and break here
                    if (checkResult != true) break;
                }
                else if (logic.CheckNextCondition == Constants.CheckNext.IfFalse)
                {
                    // User has defined that proceed to next 
                    // condition check only if the current
                    // condition is turned to False.

                    // So if not matched with what the user
                    // defined just don't proceed and break here
                    if (checkResult != false) break;
                }
                #endregion
            }

            return IsUpdated;
        }

        private ConditionalTest.TestDataType ParseTestDataType(string value)
        {
            switch (value)
            {
                case Constants.ConditionValueTypeTitle.String:
                    return ConditionalTest.TestDataType.String;

                case Constants.ConditionValueTypeTitle.Numeric:
                    return ConditionalTest.TestDataType.Numeric;

                case Constants.ConditionValueTypeTitle.DateTime:
                    return ConditionalTest.TestDataType.DateTime;

                default:
                    return ConditionalTest.TestDataType.None;

            }
        }

        private List<LogicInfo> ParseToLogicInfoCollection(ListItemReader reader)
        {

            // Caml query format to query user defined logic from
            // list, ordering by Periority field value
            string camlFormat = "<Where>" +
                                   "<And>" +
                                   "<Eq>" +
                                     "<FieldRef Name='{0}'/>" +
                                     "<Value Type='Text'>{1}</Value>" +
                                  "</Eq>" +
                                   "<Eq>" +
                                     "<FieldRef Name='{3}'/>" +
                                     "<Value Type='Boolean'>1</Value>" + // In CAML query 1=True & 0=False
                                  "</Eq>" +
                                  "</And>" +
                               "</Where>" +
                               "<OrderBy><FieldRef Name='{2}'/></OrderBy>";


            // caml query parameter values

            object[] prms =    {
                                    LogicList.Fields[Constants.LogicListFieldName.ListName].InternalName,
                                    SPEncode.HtmlEncode(reader.ItemList.Title),
                                    LogicList.Fields[Constants.LogicListFieldName.Order].InternalName,
                                    LogicList.Fields[Constants.LogicListFieldName.Active].InternalName
                                };

            var query = new SPQuery();
            query.Query = string.Format(camlFormat, prms);

            var logicItems = LogicList.GetItems(query);

            // check if logic exists
            if (logicItems.Count == 0) return null;

            var logics = new List<LogicInfo>();
            var fieldConditionParameter = LogicList.Fields[Constants.LogicListFieldName.ConditionParameter] as SPFieldMultiChoice;
            foreach (SPListItem logicItem in logicItems)
            {
                var curlogic = new LogicInfo();

                // Get the condtion
                curlogic.Condition = ConditionalTest.ParseCondition(logicItem[Constants.LogicListFieldName.Condition].ToString());

                // Get value type
                curlogic.ValueType = ParseTestDataType(logicItem[Constants.LogicListFieldName.ValueType].ToString());

                // Get the value to match by reading the field name defined by user
                curlogic.ValueToMatch = reader.GetFieldValue(logicItem[Constants.LogicListFieldName.FieldToCompare].ToString());

                // Get the value to be matched with.
                // This value may contain the field name so we need to
                // replace with those with acutal values
                object valueToCompareWith = logicItem[Constants.LogicListFieldName.ValueToComapreWith];
                if (valueToCompareWith != null)
                {
                    curlogic.ValueToMatchWith = ReplaceFieldValues(
                                                 valueToCompareWith.ToString(),
                                                 reader
                                                 );
                }

                // Get the name of field which needs to be updated
                curlogic.FieldNameToUpdate = logicItem[Constants.LogicListFieldName.UpdateField].ToString();

                // Get the value by the wich the target field to be updated with
                object updateWithField = logicItem[Constants.LogicListFieldName.UpdateWith];
                if (updateWithField == null)
                {
                    curlogic.ValueToUpdateWith = null;
                }
                else
                {
                    curlogic.ValueToUpdateWith = ReplaceFieldValues(
                                                     updateWithField.ToString(),
                                                     reader);
                }

                // Get the value on how to proceed to next condition
                curlogic.CheckNextCondition = logicItem[Constants.LogicListFieldName.CheckNext].ToString();

                // Get the periority of condition
                curlogic.Periority = Convert.ToInt32(logicItem[Constants.LogicListFieldName.Order]);

                // Get the status of the condition.
                curlogic.Active = Convert.ToBoolean(logicItem[Constants.LogicListFieldName.Active]);

                // Get selected events
                curlogic.Events = Utils.SPMultiChoiceFieldValueToStringList(logicItem[Constants.LogicListFieldName.CheckEvent]);

                // Get test options
                var selectedTestOptions = fieldConditionParameter.GetFieldValueAsText(logicItem[Constants.LogicListFieldName.ConditionParameter]);
                curlogic.TestOptions = ConditionParameters.Parse(ref selectedTestOptions);
                logics.Add(curlogic);

            }

            return logics;
        }

        private string ReplaceFieldValues(string source,ListItemReader reader)
        {
            // Stringbuilder holding final string
            // to be returned
            var sb = new StringBuilder(source);

            var placeHolderFields = Utils.GetSquareBracketPlaceHolders(ref source, false);

            foreach (string fieldName in placeHolderFields)
            {
                // Check if field exist, if not, move to next item  
                object fieldData;
                fieldData = reader.GetFieldValue(fieldName);
                if (fieldData == null)
                {
                    continue;
                }

                sb.Replace("[" + fieldName + "]", fieldData.ToString());
            }

            // Replace "[[" and "]]" with "[" and "]" respectively
            // becuase those are escape character
            sb.Replace("[[", "[");
            sb.Replace("]]", "]");

            return sb.ToString();
        }


    }
}
