﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.SharePoint;
namespace TechAmit.SharePoint.LogicalFieldUpdate
{
    public class LogicValidator
    {
        MsgReader _msgRreader;
        public LogicValidator(SPListItem logicItem)
        {
            LogicListItem = logicItem;
            Web = logicItem.Web;
            LogicList = logicItem.ParentList;
            ErrorMessages = new List<string>();
        }

        public LogicValidator(SPItemEventProperties prop)
        {
            LogicEventProperties = prop;
            Web = prop.Web;
            LogicList = prop.List;
            ErrorMessages = new List<string>();
        }

        public string LogicItemReadFieldValue(string fieldName, bool convertNullValueToEmptyString)
        {
            object value;
            if (LogicEventProperties != null)
            {
                // Read using event properties
                string fldInterName =  LogicEventProperties.List.Fields[fieldName].InternalName;
                value = LogicEventProperties.AfterProperties[fldInterName];
            }
            else
            {
                // Read using list item
                value = LogicListItem[fieldName];
            }

            if (convertNullValueToEmptyString)
            {
                return (value == null) ? string.Empty : value.ToString();
            }
            else
            {
                return (value == null) ? null : value.ToString();
            }

        }
        private SPItemEventProperties LogicEventProperties { get; set;}
        private SPListItem LogicListItem {get; set;}
        private SPList LogicList { get; set; }
        //private SPListItem LogicItem { get; set; }

        //private SPList TargetList { get; set; }
        private SPWeb Web { get; set; }
        private MsgReader MsgReader
        {
            get
            {
                if (_msgRreader == null)
                {
                    _msgRreader = new MsgReader(new MsgReader.DelgRead(GetMsg));
                }
                return _msgRreader;
            }
        }
        public List<String> ErrorMessages { get; private set; }

        // Validate logic item
        public bool Validate()
        {
            #region Check If List Exists
            string targetListTitle = LogicItemReadFieldValue(Constants.LogicListFieldName.ListName, true);
            var targetList = Web.Lists.TryGetList(targetListTitle);
            if (targetList == null)
            {
                // Show error message
                ErrorMessages.Add(MsgReader.GetById(1,targetListTitle));

                // Return
                return false;
            }
            #endregion 

            #region Check If Field To Compare Exists
            // Get the name of field (exists in target list ) that needs to be compared.
            string valueFieldToCompare = LogicItemReadFieldValue(Constants.LogicListFieldName.FieldToCompare, true);
            if (targetList.Fields.ContainsField(valueFieldToCompare)==false)
            {
                // Add error message
                ErrorMessages.Add(MsgReader.GetById(3,valueFieldToCompare,Constants.LogicListFieldName.FieldToCompare));
                return false;
            }
            #endregion

            #region Check If Update Field Exists
            // Get the name of field (exists in target list) that need to be updated
            string valueFieldToUpdate = LogicItemReadFieldValue(Constants.LogicListFieldName.UpdateField, true);
            if (targetList.Fields.ContainsField(valueFieldToUpdate) == false)
            {
                // Add error message
                ErrorMessages.Add(MsgReader.GetById(3, valueFieldToUpdate, Constants.LogicListFieldName.UpdateField));
                return false;
            }
            #endregion

            #region Check if fields exists in "Value to compare with" field
            var fieldsNotExistInValueToCompareWithField = FindPlaceHolderFieldNotExists(Constants.LogicListFieldName.ValueToComapreWith, targetList);
            if (fieldsNotExistInValueToCompareWithField.Count > 0)
            {
                // There are some fields that do not exist
                // Add error message
                ErrorMessages.Add(MsgReader.GetById(2, Constants.LogicListFieldName.ValueToComapreWith, 
                                    string.Join(" ,", fieldsNotExistInValueToCompareWithField.ToArray())));
                return false;
            }
            #endregion

            #region Check if fields exists in "Update With" field
            var fieldsNotExistInUpdateWithField = FindPlaceHolderFieldNotExists(Constants.LogicListFieldName.UpdateWith,targetList);
            if (fieldsNotExistInUpdateWithField.Count > 0)
            {
                // There are some fields that do not exists
                // Add error message
                ErrorMessages.Add(MsgReader.GetById(2, Constants.LogicListFieldName.UpdateWith,
                                    string.Join(" ,", fieldsNotExistInUpdateWithField.ToArray())));
                return false;
            }
            #endregion

            return true;
        }

        private List<string> FindPlaceHolderFieldNotExists(string fieldName, SPList targetList)
        {

            // Collection of fields that do not exist
            var fieldsNotExist = new List<string>();

            // Get the value of field that need to be checked
            object valueToCompareWith = LogicItemReadFieldValue(fieldName,false);

            if (valueToCompareWith == null) return fieldsNotExist;

            // field value contents that includes field name place holders
            string fieldValue = valueToCompareWith.ToString();

            // Get field names
            List<String> fieldColl = Utils.GetSquareBracketPlaceHolders(ref fieldValue, false);

            // Get unique field names
            List<string> uniqueFieldColl = fieldColl.Distinct().ToList();

            // Enumerate through each unique field
            foreach (string field in uniqueFieldColl)
            {
                if (targetList.Fields.ContainsField(field) == false)
                {
                    fieldsNotExist.Add(field);
                }
            }

            // Return fields that do not exists
            return fieldsNotExist;
        }

        private string GetMsg(int id)
        {
            switch (id)
            {
                case 1:
                    return "The list '{0}' does not exists";

                case 2:
                    return "Some fields that are defined in '{0}' do not exists, those fields are: {1}";

                case 3:
                    return "The field '{0}' defined for '{1}' does not exist.";

                default:
                    return string.Empty;
            }
        }
    }
}
